@@ -1,7 +1,5 @@
 
-__version__ = '0.0.0'
-# from .version import __version__
-
+__version__ = '0.0.1'
 
 
 try:
@@ -10,18 +8,25 @@ except ImportError:
     def execute_model(args):
         return max(args, key=len)
 
+
 try:
     import jaxlib
 except ImportError:
+    import numpy
     COMPILER = None
+    BACKEND = numpy
 else:
+    import jax
     COMPILER = jaxlib
+    BACKEND = jax.numpy
 
 
-# from .core import *
-# import anon.io
+import anon.conf
+import anon.core
 import anon.diff
 import anon.dual
 import anon.quad
+import anon.lila
+import anon.step
+
 
-# import anabel.ops
