@@ -1,13 +1,18 @@
+
+
 try:
-    from jax.config import *
+    from jax.config import config
 except:
-    pass
+    config = {}
 
 _BACKEND = {
     'ops': 'jax',
     'numeric': 'jax'
 }
 
+
+#config.update('jax_disable_jit', True)
+
 def use(backend, **kwds):
     _BACKEND['ops'] = backend
     _BACKEND['numeric'] = backend
