@@ -556,14 +556,20 @@ def _iter_fn(self):
     return iter(asdict(self))
 
 def _keys_fn(self):
-    return [key for key in self]
+    return [key for key in self if key is not ...]
 
 def _items_fn(self):
-    return tuple(v for v in asdict(self).items())
+    return tuple(v for v in asdict(self).items() if v[0] is not ...)
 
 def _values_fn(self):
     return tuple(v for v in asdict(self).values())
 
+def _get_item_fn(self, name):
+    if name is ...:
+        return self._child
+    else:
+        return getattr(self,name)
+
 def _frozen_get_del_attr(cls, fields, globals):
     locals = {'cls': cls,
               'FrozenInstanceError': FrozenInstanceError}
@@ -692,6 +698,10 @@ def _get_field(cls, a_name, a_type):
 
     # If the default value isn't derived from Field, then it's only a
     # normal default value.  Convert it to a Field().
+    # if a_name is ...:
+    #     default = MISSING
+    # else:
+    #     default = getattr(cls, a_name, MISSING)
     default = getattr(cls, a_name, MISSING)
     if isinstance(default, Field):
         f = default
@@ -992,7 +1002,7 @@ def _process_class(cls, init, repr, eq, order, unsafe_hash, frozen):
     _set_new_attribute(cls,'keys',_keys_fn)
     _set_new_attribute(cls,'items',_items_fn)
     _set_new_attribute(cls,'values',_values_fn)
-    _set_new_attribute(cls,'__getitem__',lambda *args: getattr(*args))
+    _set_new_attribute(cls,'__getitem__',_get_item_fn)
 
     if frozen:
         for fn in _frozen_get_del_attr(cls, field_list, globals):
@@ -1018,7 +1028,7 @@ def _process_class(cls, init, repr, eq, order, unsafe_hash, frozen):
     return cls
 
 
-def interface(cls=None, /, *, init=True, repr=True, eq=True, order=False,
+def interface(cls=None,  *, init=True, repr=True, eq=True, order=False,
               unsafe_hash=False, frozen=False):
     """Returns the same class as was passed in, with dunder methods
     added based on the fields defined in the class.
@@ -1185,15 +1195,19 @@ def new_interface(cls_name, fields, *, bases=(), namespace=None, init=True,
     The interface name will be 'cls_name'.  'fields' is an iterable
     of either (name), (name, type) or (name, type, Field) objects. If type is
     omitted, use the string 'typing.Any'.  Field objects are created by
-    the equivalent of calling 'field(name, type [, Field-info])'.
-      C = new_interface('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))
+    the equivalent of calling `field(name, type [, Field-info])`.
+    ```python
+    C = new_interface('C', ['x', ('y', int), ('z', int, field(init=False))], bases=(Base,))
+    ```
     is equivalent to:
-      @interface
-      class C(Base):
-          x: 'typing.Any'
-          y: int
-          z: int = field(init=False)
-    For the bases and namespace parameters, see the builtin type() function.
+    ```python
+    @interface
+    class C(Base):
+        x: 'typing.Any'
+        y: int
+        z: int = field(init=False)
+    ```
+    For the bases and namespace parameters, see the builtin `type()` function.
     The parameters init, repr, eq, order, unsafe_hash, and frozen are passed to
     interface().
     """
@@ -1212,6 +1226,10 @@ def new_interface(cls_name, fields, *, bases=(), namespace=None, init=True,
         if isinstance(item, str):
             name = item
             tp = 'typing.Any'
+        elif item is ...:
+            continue
+            # name = ...
+            # tp = 'typing.Any'
         elif len(item) == 2:
             name, tp, = item
         elif len(item) == 3:
@@ -1220,8 +1238,9 @@ def new_interface(cls_name, fields, *, bases=(), namespace=None, init=True,
         else:
             raise TypeError(f'Invalid field: {item!r}')
 
-        if not isinstance(name, str) or not name.isidentifier():
+        if not isinstance(name, str) or (not name.isidentifier()):
             raise TypeError(f'Field names must be valid identifiers: {name!r}')
+
         if keyword.iskeyword(name):
             raise TypeError(f'Field names must not be keywords: {name!r}')
         if name in seen:
@@ -1234,6 +1253,8 @@ def new_interface(cls_name, fields, *, bases=(), namespace=None, init=True,
     # We use `types.new_class()` instead of simply `type()` to allow dynamic creation
     # of generic interfaceses.
     cls = types.new_class(cls_name, bases, {}, lambda ns: ns.update(namespace))
+    if ... in fields:
+        cls._child = fields[...]
     return interface(cls, init=init, repr=repr, eq=eq, order=order,
                      unsafe_hash=unsafe_hash, frozen=frozen)
 
