@@ -1,12 +1,33 @@
+"""Utilities for performing quadrature.
+
+
+Sources
+
+- @abramowitz2013handbook
+- @casio2021guassian
+- https://pomax.github.io/bezierinfo/legendre-gauss.html
+
+"""
+
 import math
-from typing import Final
 from functools import partial
-import jax
-#import emx as emx 
-import jax.numpy as jnp
+try:
+    from typing import Final, Callable
+except:
+    Final, Callable = object(), object()
 import numpy as np
+try:
+    import jax
+    #import emx as emx 
+    scipy.special = jax.scipy.special
+    import jax.numpy as jnp
+except:
+    import scipy.special
+    jnp = np
+
+from . import quad_simplex as simplex
+
 
-## < Quadrature
 
 _quad_rules: Final = {
     # Closed Newton-Cotes.  deg(P) = n−1
@@ -28,37 +49,67 @@ _quad_rules: Final = {
     'hermite':   {'deg': lambda n: None},
 }
 
+class QuadratureScheme:
+    def __init__(self, degree, points, weights):
+        self.degree  = degree
+        self.points  = points
+        self.weights = weights
 
-def quad_points(n=None, deg=None, bounds=(-1.,1.), fam=None, closed=None, even=False, rule='mid', interpolator=None, error_term=False):
-    """
 
-    Parameters:
-        n: number of sampling points
+def _read_m228(filename):
+    data = anp.asarray(np.loadtxt(filename))
+    points = data[:,:2]
+    weights = data[:,2]
+    return points, weights
 
-        deg: degree
 
-    Examples:
+def load(filename, file_format=None):
+
+    pass
+
+def quad_points(n=None, deg=None, bounds=(-1.,1.), fam:str=None, closed=None, even=False, rule:str='mid', interpolator=None, error_term=False):
+    """Generate weights and locations for a given quadrature scheme
+
+    Parameters
+    ----------
+    n: int
+        number of sampling points
+    deg: int
+        degree of highest order polynomial exactly integrated
+    closed:
+        True if endpoints are included
+
+
+    Examples
+    --------
 
     Midpoint rule:
-        xi, wi =  quad_points(deg=1, rule='mid')
-
+    ```python
+    >>> xi, wi =  quad_points(deg=1, rule='mid')
+    ```
     Trapezoidal rule:
-        xi, wi =  quad_points(deg=1, even=True)
-
-
+    ```python
+    >>> xi, wi =  quad_points(deg=1, even=True)
+    ```
     Composite trapezoidal rule:
-        xi, wi = quad_points(n=4, deg=1, rule='cotes')
-    
+    ```python
+    >>> xi, wi = quad_points(n=4, deg=1, rule='cotes')
+    ``` 
     Composite Simpson's rule 
-        xi, wi = quad_points(n=6, deg=2, rule='cotes')
+    ```python
+    >>> xi, wi = quad_points(n=6, deg=2, rule='cotes')
+    ```
+    Gauss-Legendre
+    ```python
+    >>> xi, wi = quad_points(n=4, rule='legendre')
+    ```
 
-    Gauss-legendre
-        xi, wi = quad_points(n=4, rule='legendre')
+    Dependents
+    ----------
 
-    'closed': True if endpoints are included
-    'deg': degree of highest order polynomial exactly integrated
-    'n0': minimum number of points ; For Newton-Cotes rules, this is the 
-          number of points for the base (non-composite rule)
+    Studies
+    -------
+    [elle-0005](/stdy/elle-0005)
 
     """
     if 'trap' in rule:
@@ -108,6 +159,10 @@ def quad_points(n=None, deg=None, bounds=(-1.,1.), fam=None, closed=None, even=F
                 dxi[i*deg:(i+1)*deg+1] = dxi[i*deg:(i+1)*deg+1] + wi
 
             return xi,dxi/nsums
+
+    elif "m228" in rule:
+        pass
+
     else: # Gaussian
         if n is None: n = _quad_rules[rule]['n'](deg)
         x0, x1 = bounds
@@ -198,17 +253,20 @@ def nquad_points(n, DX=None, X=None, family='gauss-legendre',jac=None):
 
 
 def newton_cotes(rn, equal=True, error_term=False):
-    r"""
-    Return weights and error coefficient for Newton-Cotes integration.
+    r"""Return weights and error coefficient for Newton-Cotes integration.
+
+    **This function is forked from the `scipy` library**
+    
     Suppose we have (N+1) samples of f at the positions
-    x_0, x_1, ..., x_N.  Then an N-point Newton-Cotes formula for the
-    integral between x_0 and x_N is:
-    :math:`\int_{x_0}^{x_N} f(x)dx = \Delta x \sum_{i=0}^{N} a_i f(x_i)
-    + B_N (\Delta x)^{N+2} f^{N+1} (\xi)`
-    where :math:`\xi \in [x_0,x_N]`
-    and :math:`\Delta x = \frac{x_N-x_0}{N}` is the average samples spacing.
+    `x_0, x_1, ..., x_N`.  Then an N-point Newton-Cotes formula for the
+    integral between `x_0` and `x_N` is:
+    $$\int_{x_0}^{x_N} f(x)dx = \Delta x \sum_{i=0}^{N} a_i f(x_i)
+    + B_N (\Delta x)^{N+2} f^{N+1} (\xi)$$
+    where $\xi \in [x_0,x_N]$
+    and $\Delta x = \frac{x_N-x_0}{N}$ is the average samples spacing.
     If the samples are equally-spaced and N is even, then the error
-    term is :math:`B_N (\Delta x)^{N+3} f^{N+2}(\xi)`.
+    term is $B_N (\Delta x)^{N+3} f^{N+2}(\xi)$.
+    
     Parameters
     ----------
     rn : int
@@ -217,6 +275,7 @@ def newton_cotes(rn, equal=True, error_term=False):
         the length of `rn`.  N is the order of the Newton-Cotes integration.
     equal : int, optional
         Set to 1 to enforce equally spaced data.
+    
     Returns
     -------
     an : ndarray
@@ -224,9 +283,10 @@ def newton_cotes(rn, equal=True, error_term=False):
         positions.
     B : float
         Error coefficient.
+    
     Examples
     --------
-    Compute the integral of sin(x) in [0, :math:`\pi`]:
+    Compute the integral of $\sin(x)$ in $[0, \pi]$:
     >>> from scipy.integrate import newton_cotes
     >>> def f(x):
     ...     return np.sin(x)
@@ -246,6 +306,7 @@ def newton_cotes(rn, equal=True, error_term=False):
      6   2.000017814   1.78136e-05
      8   1.999999835   1.64725e-07
     10   2.000000001   1.14677e-09
+
     Notes
     -----
     Normally, the Newton-Cotes rules are used on smaller integration
@@ -292,17 +353,18 @@ def newton_cotes(rn, equal=True, error_term=False):
 
     BN = BN - jnp.dot(yi**power, ai)
     p1 = power+1
-    fac = power*jnp.log(N) - jax.scipy.special.gammaln(p1)
+    fac = power*jnp.log(N) - scipy.special.gammaln(p1)
     fac = jnp.exp(fac)
     if error_term: return ai, BN*fac
     return ai
 
-def gauss_quad(f, n: int, bounds=(-1,1), family='legendre'):
-    """https://keisan.casio.com/exec/system/1329114617
+def gauss_quad(f:Callable, n: int, bounds:tuple=(-1,1), family='legendre'):
+    """Integrate a function using Gaussian quadrature.
     
-    Other sources:
-    - https://pomax.github.io/bezierinfo/legendre-gauss.html
-    - 
+    Parameters
+    ----------
+    f: Callable
+    family: str
     """
 
     x,w = GAUSS_POINTS[family][n-1].T
@@ -312,125 +374,15 @@ def gauss_quad(f, n: int, bounds=(-1,1), family='legendre'):
 def gauss_points(n: int, bounds=(-1,1), family='legendre'):
     """
     
-    n: 
+    Parameters
+    ----------
+    n: int
     
-    
-    https://keisan.casio.com/exec/system/1329114617
-    
-    Other sources:
-    - https://pomax.github.io/bezierinfo/legendre-gauss.html
-    - 
     """
 
     x, w = GAUSS_POINTS[family][n-1].T
     return x, w
 
-
-class iHLRF:
-    """Improved Hasofer/Lind - Rackwitz/Fiessler algorithm for constrained
-    minimization.
-    
-    """
-    def __init__(self, f, gradf, u0, loss=None, tol1=1.0e-4, tol2=1.0e-4, maxiter=20, maxiter_step=20):
-        self.u0 = u0
-        self.f = f 
-        self.gradf = gradf
-        self.tol1 = tol1
-        self.tol2 = tol2
-        self.maxiter = maxiter
-        self.maxiter_step = maxiter_step
-        self.loss = loss
-        if loss is None: 
-            self.loss = jnp.linalg.norm
-
-        # self.init()
-
-    def init(self,verbose=False):
-        self.G0 = self.f(self.u0) # scale parameter
-        self.ui = self.u0[None,:].T
-        self.Gi = self.f(self.ui[:,0].T)
-        self.GradGi = self.gradf(self.ui[:,0].T)
-        self.alphai = -(self.GradGi / jnp.linalg.norm(self.GradGi))[None,:]
-        self.count = 0
-        self.spec1 = abs(self.Gi/self.G0)
-        self.spec2 = self.loss(self.ui - (self.alphai@self.ui)*self.alphai.T )
-
-        if verbose:
-            print('\niHL-RF Algorithm (Zhang and Der Kiureghian 1995)***************',
-                  '\nInitialize iHL-RF: \n',
-                  'u0: ', np.around(self.u0.T,4), '\n',
-                  'G0: ',np.around(self.G0,4),'\n',
-                  'ui: ', np.around(self.ui.T,4), '\n',
-                  'Gi: ', np.around(self.Gi,4), '\n',
-                  'GradGi: ', np.around(self.GradGi,4),'\n',
-                  'alphai: ', np.around(self.alphai,4),'\n',
-                  'spec1: ' , np.around(self.spec1,4),'\n',
-                  'spec2: ' , np.around(self.spec2,4),'\n',)
-    
-    def incr(self,basic=False, verbose=False):
-        if basic == True:
-            self.ui1 = self.ui + self.lamda * self.di
-            return self.ui1 
-
-        self.ci = self.loss(self.ui) / jnp.linalg.norm(self.GradGi) + 10.0
-        self.mi = 0.5*self.loss(self.ui)**2 + self.ci*abs(self.Gi)
-        self.mi1 = 0.5*self.loss(self.ui1)**2 + self.ci*abs(self.Gi1)
-
-        self.count_step = 0
-        while (self.mi1 >= self.mi) and (self.count_step <= self.maxiter_step):
-            self.lamda = self.lamda/2
-            self.ui1 = self.ui + self.lamda * self.di
-            self.Gi1 = self.f(self.ui1[:,0].T)
-            self.mi1 = 0.5*np.linalg.norm(self.ui1)**2 + self.ci*abs(self.Gi1)
-            self.count_step += 1
-
-        return self.ui1
-    
-    def dirn(self):
-        self.di = (self.Gi/jnp.linalg.norm(self.GradGi) + self.alphai@self.ui)*self.alphai.T - self.ui
-        return self.di
-
-    def step(self,verbose=False,basic=False):
-        # self.di = (self.Gi/np.linalg.norm(self.GradGi) + self.alphai@self.ui)*self.alphai.T - self.ui
-        di = self.dirn()
-        self.lamda = 1.0
-        self.ui1 = self.ui + self.lamda * di
-        self.Gi1 = self.f(self.ui1[:,0].T)
-
-        ui1 = self.incr(basic)
-
-        self.ui = ui1  
-        self.Gi = self.f(self.ui[:,0].T)
-        self.GradGi = self.gradf(self.ui[:,0].T)
-        self.alphai = -(self.GradGi / jnp.linalg.norm(self.GradGi))[None,:]
-
-        self.spec1 = abs(self.Gi/self.G0)
-        self.spec2 = self.loss(self.ui - (self.alphai@self.ui)*self.alphai.T)
-        self.count += 1
-
-
-        if verbose: print('\niHL-RF step: {}'.format(self.count))
-
-        if verbose:
-            print('ui: ', '\n', np.around(self.ui,4), '\n',
-                  'Gi: ',       np.around(self.Gi,4), '\n',
-                  'GradGi: ', np.around(self.GradGi,4),'\n',
-                  'alphai: ', np.around(self.alphai,4),'\n',
-                  'spec1: ' , np.around(self.spec1,4),'\n',
-                  'spec2: ' , np.around(self.spec2,4),'\n',)
-        return self.ui
-    
-    def run(self,verbose=False, steps=None):
-        self.init(verbose=verbose)
-        if steps is not None: self.maxiter = steps
-        while not(self.spec1 < self.tol1 and self.spec2 < self.tol2):
-            self.step(verbose=verbose)
-
-            if (self.count > self.maxiter): break
-
-        return self.ui
-
-
 GAUSS_POINTS: Final = {
     'legendre':[
         np.array([[0., 2.],]),
